// Incrementing OFFLINE_VERSION will kick off the install event and force
// previously cached resources to be updated from the network.
// This variable is intentionally declared and unused.
// Add a comment for your linter if you want:
// eslint-disable-next-line no-unused-vars
const OFFLINE_VERSION = {VERSION};
const CACHE_NAME = "{CACHE_NAME}";
const OFFLINE_URL = "offline.html";
const STRATEGY = "{STRATEGY}";
const PERIODIC_SYNC = "{PERIODIC_SYNC}";
const BACKGROUND_SYNC = "{BACKGROUND_SYNC}";
const DEBUG = "{DEBUG}";
// Cached resources that match the following strings should not be periodically updated.
// These are the tips html pages themselves, and their images.
// Everything else, we try to update on a regular basis, to make sure lists of tips get updated and css/js are recent too.
const DONT_UPDATE_RESOURCES = [];

self.addEventListener("install", (event) => {
  event.waitUntil(
    (async () => {
      const cache = await caches.open(CACHE_NAME);
      // Setting {cache: 'reload'} in the new request will ensure that the
      // response isn't fulfilled from the HTTP cache; i.e., it will be from
      // the network.
      await cache.add(new Request(OFFLINE_URL, {cache: "reload"}));
      await cache.add(new Request("{OFFLINE_IMAGE_URL}", {cache: "reload"}));
      await cache.addAll({CACHE_ASSETS});
    })()
  );
  
  // Force the waiting service worker to become the active service worker.
  self.skipWaiting();
});


// Enable navigation preloads to speedup loading
addEventListener('activate', event => {
  event.waitUntil(async function() {
    if (self.registration.navigationPreload) {
      pwaConsoleLog('[PWA] Navigation preloads has been enabled.');
      await self.registration.navigationPreload.enable();
    }
  }());
});


// Remove old cache we do not need
self.addEventListener('activate', (e) => {
  e.waitUntil(caches.keys().then((keyList) => {
    return Promise.all(keyList.map((key) => {
      if (key === CACHE_NAME) { return; }
      return caches.delete(key);
    }))
  }));
});


// We have a cache-first strategy, where we look for resources in the cache first
// and only on the network if this fails.
// We also periodically update the cache in the background for the main pages.
// STRATEGY: CACHE FIRST
if(STRATEGY == 'CACHE-FIRST') {
  self.addEventListener("fetch", (event) => {
    // We only want to call event.respondWith() if this is a navigation request
    // for an HTML page.
    if (event.request.mode === "navigate" && event.request.method === "GET" && event.request.url.indexOf('/{OC_ADMIN_DIR}/') === -1) {
      event.respondWith(
        (async () => {
          try {
            // First, respond from the cache if we can
            const cachedResponse = await caches.match(event.request);
            if (cachedResponse) {
              return cachedResponse;
            }
    
    
            // Next, try to use the navigation preload response if it's supported.
            const preloadResponse = await event.preloadResponse;
            if (preloadResponse) {
              if(!preloadResponse || preloadResponse.status !== 200 || preloadResponse.type !== 'basic') {
                return preloadResponse;
              }
              
              let response2 = preloadResponse.clone();

              if (!event.request.url.includes('google') && !event.request.url.includes('adservice') && !event.request.url.includes('browser-sync') && !event.request.url.includes('{OC_ADMIN_DIR}')) {
               event.waitUntil(
                  caches.open(CACHE_NAME).then(cache => {
                    cache.put(event.request.url, response2)
                  })
                );
              }
              
              return preloadResponse;
            }
            
            
            // Last, try network
            const networkResponse = await fetch(event.request);
            if(networkResponse) {
              // Wait until the response we received is added to the cache.
              if(!networkResponse || networkResponse.status !== 200 || networkResponse.type !== 'basic') {
                return networkResponse;
              }
              
              let response3 = networkResponse.clone();

              if (!event.request.url.includes('google') && !event.request.url.includes('adservice') && !event.request.url.includes('browser-sync') && !event.request.url.includes('{OC_ADMIN_DIR}')) {
               event.waitUntil(
                  caches.open(CACHE_NAME).then(cache => {
                    cache.put(event.request.url, response3)
                  })
                );
              }

              return networkResponse;
            }
          } catch (error) {
            // catch is only triggered if an exception is thrown, which is likely
            // due to a network error.
            // If fetch() returns a valid HTTP response with a response code in
            // the 4xx or 5xx range, the catch() will NOT be called.

            // Check the cache first to see if the asset exists, and if it does, 
            // return the cached asset.
            pwaConsoleLog("[PWA] Fetch failed, returning cache or offline asset instead.");

            const cache = await caches.open(CACHE_NAME);
            const cachedResponse = await cache.match(event.request.url.split('?v=')[0], {"ignoreSearch": true});
            
            if(cachedResponse) {
              return cachedResponse;
            }
          }
        })()
      );
    } else if (event.request.url.indexOf('/{OC_ADMIN_DIR}/') === -1) {
      // There should fallback retrieving resources like css, js etc and check them in cache
      event.respondWith(
        (async () => {
          try {
            const preloadResponse = await event.preloadResponse;
            if(preloadResponse) {
              return preloadResponse;
            }

            const networkResponse = await fetch(event.request);
            if(networkResponse) {
              return networkResponse;
            }
          } catch (error) {
            // catch is only triggered if an exception is thrown, which is likely
            // due to a network error.
            // If fetch() returns a valid HTTP response with a response code in
            // the 4xx or 5xx range, the catch() will NOT be called.

            // Check the cache first to see if the asset exists, and if it does, 
            // return the cached asset.
            pwaConsoleLog("[PWA] Fetch failed, returning cache or offline asset instead.");

            const cache = await caches.open(CACHE_NAME);
            const cachedResponse = await cache.match(event.request.url.split('?v=')[0], {"ignoreSearch": true});
            
            if(cachedResponse) {
              return cachedResponse;
            }
          }
        })()
      );
    }
  });
}

// Older version of cache first, but functional
if(STRATEGY == 'CACHE-FIRST-BACKUP') {
  self.addEventListener("fetch", event => {
    const request = event.request;
    const url = request.url;

    // If we are requesting an HTML page.
    if (event.request.mode === "navigate" && event.request.method === "GET" && event.request.url.indexOf('/{OC_ADMIN_DIR}/') === -1) {
      event.respondWith(
        // Check the cache first to see if the asset exists, and if it does, 
        // return the cached asset.
        caches.match(request.url).then(cachedResult => {
          if (cachedResult) {
            return cachedResult;
          }
          
          // If the asset isn't in the cache, fall back to a network request 
          // for the asset, and proceed to cache the result.
          return fetch(request).then(response => {
            // Wait until the response we received is added to the cache.
            if(!response || response.status !== 200 || response.type !== 'basic') {
              return response;
            }
            
            let response2 = response.clone();

            if (!event.request.url.includes('google') && !event.request.url.includes('adservice') && !event.request.url.includes('browser-sync') && !event.request.url.includes('{OC_ADMIN_DIR}')) {
              event.waitUntil(
                caches.open(CACHE_NAME).then(cache => {
                  return cache.put(request.url, response2)
                })
              );
            }
            
            return response;
          })
          // If the network is unavailable to make a request, pull the offline
          // page out of the cache.
          .catch(() => caches.match(OFFLINE_URL));
        })
      ); // end respondWith
    } // end if HTML
  });
}


// STRATEGY: NETWORK FIRST
// with caching
if(STRATEGY == 'NETWORK-FIRST') {
  self.addEventListener("fetch", (event) => {
    // We only want to call event.respondWith() if this is a navigation request
    // for an HTML page.
    if (event.request.mode === "navigate" && event.request.method === "GET" && event.request.url.indexOf('/{OC_ADMIN_DIR}/') === -1) {
      event.respondWith(
        (async () => {
          try {
            // First, try to use the navigation preload response if it's supported.
            const preloadResponse = await event.preloadResponse;
            
            if (preloadResponse) {
              if(!preloadResponse || preloadResponse.status !== 200 || preloadResponse.type !== 'basic') {
                return preloadResponse;
              }
              
              let response2 = preloadResponse.clone();

              if (!event.request.url.includes('google') && !event.request.url.includes('adservice') && !event.request.url.includes('browser-sync') && !event.request.url.includes('{OC_ADMIN_DIR}')) {
               event.waitUntil(
                  caches.open(CACHE_NAME).then(cache => {
                    cache.put(event.request.url, response2)
                  })
                );
              }
              
              return preloadResponse;
            }

            // Always try the network first, cache response for offline experience
            return fetch(event.request).then(response => {
              // Wait until the response we received is added to the cache.
              if(!response || response.status !== 200 || response.type !== 'basic') {
                return response;
              }
              
              let response3 = response.clone();

              if (!event.request.url.includes('google') && !event.request.url.includes('adservice') && !event.request.url.includes('browser-sync') && !event.request.url.includes('{OC_ADMIN_DIR}')) {
               event.waitUntil(
                  caches.open(CACHE_NAME).then(cache => {
                    cache.put(event.request.url, response3)
                  })
                );
              }

              return response;
            });            

          } catch (error) {
            // catch is only triggered if an exception is thrown, which is likely
            // due to a network error.
            // If fetch() returns a valid HTTP response with a response code in
            // the 4xx or 5xx range, the catch() will NOT be called.

            // Check the cache first to see if the asset exists, and if it does, 
            // return the cached asset.
            pwaConsoleLog("[PWA] Fetch failed, returning cache or offline page instead.");

            const cache = await caches.open(CACHE_NAME);
            const cachedResponse = await cache.match(event.request.url);
            const offlineResponse = await cache.match(OFFLINE_URL);
            
            if(cachedResponse) {
              return cachedResponse;
            }
            
            return offlineResponse;
          }
        })()
      );
    } else if (event.request.url.indexOf('/{OC_ADMIN_DIR}/') === -1) {
      // There should fallback retrieving resources like css, js etc and check them in cache
      event.respondWith(
        (async () => {
          try {
            const preloadResponse = await event.preloadResponse;
            if(preloadResponse) {
              return preloadResponse;
            }

            const networkResponse = await fetch(event.request);
            return networkResponse;
          } catch (error) {
            // catch is only triggered if an exception is thrown, which is likely
            // due to a network error.
            // If fetch() returns a valid HTTP response with a response code in
            // the 4xx or 5xx range, the catch() will NOT be called.

            // Check the cache first to see if the asset exists, and if it does, 
            // return the cached asset.
            pwaConsoleLog("[PWA] Fetch failed, returning cache or offline asset instead.");

            const cache = await caches.open(CACHE_NAME);
            const cachedResponse = await cache.match(event.request.url.split('?v=')[0], {"ignoreSearch": true});
            return cachedResponse;
          }
        })()
      );
    }
  });
}


// STRATEGY: NETWORK FIRST
// No caching
if(STRATEGY == 'NETWORK-FIRST2') {
  self.addEventListener("fetch", (event) => {
    // We only want to call event.respondWith() if this is a navigation request
    // for an HTML page.
    if (event.request.mode === "navigate" && event.request.method === "GET" && event.request.url.indexOf('/{OC_ADMIN_DIR}/') === -1) {
      event.respondWith(
        (async () => {
          try {
            // First, try to use the navigation preload response if it's supported.
            const preloadResponse = await event.preloadResponse;
            if (preloadResponse) {
              return preloadResponse;
            }

            // Always try the network first.
            const networkResponse = await fetch(event.request);
            return networkResponse;
          } catch (error) {
            // catch is only triggered if an exception is thrown, which is likely
            // due to a network error.
            // If fetch() returns a valid HTTP response with a response code in
            // the 4xx or 5xx range, the catch() will NOT be called.
            pwaConsoleLog("[PWA] Fetch failed, returning offline page instead.");

            const cache = await caches.open(CACHE_NAME);
            const offlineResponse = await cache.match(OFFLINE_URL);
            return offlineResponse;
          }
        })()
      );
    }
  });
}


if(BACKGROUND_SYNC == '1' && STRATEGY != 'NOCACHE') {
  // Listen the background sync events to update the cached resources.
  self.addEventListener('sync', event => {
    if (event.tag === 'update-cached-content-background') {
      event.waitUntil(pwaUpdateCachedContent(event.tag));
    }
  });
}


if(PERIODIC_SYNC == '1' && STRATEGY != 'NOCACHE') {
  // Listen the periodic background sync events to update the cached resources.
  self.addEventListener('periodicsync', event => {
    if (event.tag === 'update-cached-content-periodic') {
      event.waitUntil(pwaUpdateCachedContent(event.tag));
    }
  });
}

async function pwaUpdateCachedContent($tag) {
  pwaConsoleLog('[PWA] Refreshing cache (' + $tag + ').');
  const requests = await pwaFindCacheEntriesToBeRefreshed();
  const cache = await caches.open(CACHE_NAME);
  
  for (const request of requests) {
    try {
      // Fetch the new version.
      const fetchResponse = await fetch(request);
      
      // Refresh the cache in case it meets basic criteria.
      if(!(!fetchResponse || fetchResponse.status !== 200 || fetchResponse.type !== 'basic')) {
        let fetchResponse2 = fetchResponse.clone();
        await cache.put(request, fetchResponse2);
      }
    } catch (e) {
      // Fail silently, we'll just keep whatever we already had in the cache.
    }
  }
}

// Find the entries that are already cached and that we do want to update. This way we only
// update these ones and let the user visit new pages when they are online to populate more things
// in the cache.
async function pwaFindCacheEntriesToBeRefreshed() {
  const cache = await caches.open(CACHE_NAME);
  const requests = await cache.keys();
  return requests.filter(request => {
    return !DONT_UPDATE_RESOURCES.some(pattern => request.url.includes(pattern));
  });
}

// Log to console based on preference
async function pwaConsoleLog(text) {
  if(DEBUG == "1") {
    console.log(text);
  }
}
  